MODULE bdytrc
   !!======================================================================
   !!                       ***  MODULE  bdytrc  ***
   !! Ocean tracers:   Apply boundary conditions for tracers
   !!======================================================================
   !! History :  1.0  !  2005-01  (J. Chanut, A. Sellar)  Original code
   !!            3.0  !  2008-04  (NEMO team)  add in the reference version
   !!            3.4  !  2011     (D. Storkey) rewrite in preparation for OBC-BDY merge
   !!            3.5  !  2012     (S. Mocavero, I. Epicoco) Optimization of BDY communications
   !!                    2017     (xianmin@ualberta.ca): tracer bdy for BLING
   !!----------------------------------------------------------------------
#if defined key_bdy && key_top && key_bling
   !!----------------------------------------------------------------------
   !!   'key_bdy'                     Unstructured Open Boundary Conditions
   !!----------------------------------------------------------------------
   !!   bdy_trc            : Apply open boundary conditions to bling tracers
   !!   bdy_trc_frs        : Apply Flow Relaxation Scheme
   !!----------------------------------------------------------------------
   USE timing          ! Timing
   USE oce             ! ocean dynamics and tracers variables
   USE dom_oce         ! ocean space and time domain variables 
   USE bdy_oce         ! ocean open boundary conditions
   USE bdylib          ! for orlanski library routines
   USE bdydta, ONLY:   bf
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE in_out_manager  ! I/O manager
   USE trc


   IMPLICIT NONE
   PRIVATE

   PUBLIC bdy_trc      ! routine called in trcnxt.F90 
   PUBLIC bdy_trc_dmp  ! routine called in trcstp.F90 

   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , NEMO Consortium (2010)
   !! $Id: bdytra.F90 4292 2013-11-20 16:28:04Z cetlod $ 
   !! Software governed by the CeCILL licence (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE bdy_trc( kt )
      !!----------------------------------------------------------------------
      !!                  ***  SUBROUTINE bdy_trc  ***
      !!
      !! ** Purpose : - Apply open boundary conditions for TOP tracers
      !!
      !!----------------------------------------------------------------------
      INTEGER, INTENT( in ) :: kt     ! Main time step counter
      !!
      INTEGER               :: ib_bdy ! Loop index

      DO ib_bdy=1, nb_bdy

         SELECT CASE( cn_trc(ib_bdy) )
         CASE('none')
            CYCLE
         CASE('frs')
            CALL bdy_trc_frs( idx_bdy(ib_bdy), dta_bdy(ib_bdy), kt )
         CASE('specified')
            CALL bdy_trc_spe( idx_bdy(ib_bdy), dta_bdy(ib_bdy), kt )
         CASE('neumann')
            CALL bdy_trc_nmn( idx_bdy(ib_bdy), dta_bdy(ib_bdy), kt )
         CASE('orlanski')
            CALL bdy_trc_orlanski( idx_bdy(ib_bdy), dta_bdy(ib_bdy), ll_npo=.false. )
         CASE('orlanski_npo')
            CALL bdy_trc_orlanski( idx_bdy(ib_bdy), dta_bdy(ib_bdy), ll_npo=.true. )
         CASE('runoff')
            CALL bdy_trc_rnf( idx_bdy(ib_bdy), dta_bdy(ib_bdy), kt )
         CASE DEFAULT
            CALL ctl_stop( 'bdy_trc : unrecognised option for open boundaries for top tracers' )
         END SELECT
         ! Boundary points should be updated
         CALL lbc_bdy_lnk( tra(:,:,:,1), 'T', 1., ib_bdy )
         CALL lbc_bdy_lnk( tra(:,:,:,2), 'T', 1., ib_bdy )
         CALL lbc_bdy_lnk( tra(:,:,:,3), 'T', 1., ib_bdy )
         CALL lbc_bdy_lnk( tra(:,:,:,4), 'T', 1., ib_bdy )
         CALL lbc_bdy_lnk( tra(:,:,:,5), 'T', 1., ib_bdy )
         CALL lbc_bdy_lnk( tra(:,:,:,6), 'T', 1., ib_bdy )

      ENDDO
      !

   END SUBROUTINE bdy_trc

   SUBROUTINE bdy_trc_frs( idx, dta, kt )
      !!----------------------------------------------------------------------
      !!                 ***  SUBROUTINE bdy_trc_frs  ***
      !!                    
      !! ** Purpose : Apply the Flow Relaxation Scheme for tracers at open boundaries.
      !! 
      !! Reference : Engedahl H., 1995, Tellus, 365-382.
      !!----------------------------------------------------------------------
      INTEGER,         INTENT(in) ::   kt
      TYPE(OBC_INDEX), INTENT(in) ::   idx  ! OBC indices
      TYPE(OBC_DATA),  INTENT(in) ::   dta  ! OBC external data
      !! 
      REAL(wp) ::   zwgt           ! boundary weight
      INTEGER  ::   ib, ik, igrd   ! dummy loop indices
      INTEGER  ::   ii, ij         ! 2D addresses
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 ) CALL timing_start('bdy_trc_frs')
      !
      igrd = 1                       ! Everything is at T-points here
      DO ib = 1, idx%nblen(igrd)
         DO ik = 1, jpkm1
            ii = idx%nbi(ib,igrd)
            ij = idx%nbj(ib,igrd)
            zwgt = idx%nbw(ib,igrd)
            tra(ii,ij,ik,1) = ( tra(ii,ij,ik,1) + zwgt * ( dta%po4(ib,ik) - tra(ii,ij,ik,1) ) ) * tmask(ii,ij,ik)         
            tra(ii,ij,ik,2) = ( tra(ii,ij,ik,1) + zwgt * ( dta%dop(ib,ik) - tra(ii,ij,ik,2) ) ) * tmask(ii,ij,ik)         
            tra(ii,ij,ik,3) = ( tra(ii,ij,ik,1) + zwgt * ( dta%fed(ib,ik) - tra(ii,ij,ik,3) ) ) * tmask(ii,ij,ik)         
            tra(ii,ij,ik,4) = ( tra(ii,ij,ik,1) + zwgt * ( dta%oxy(ib,ik) - tra(ii,ij,ik,4) ) ) * tmask(ii,ij,ik)         
            tra(ii,ij,ik,5) = ( tra(ii,ij,ik,1) + zwgt * ( dta%dic(ib,ik) - tra(ii,ij,ik,5) ) ) * tmask(ii,ij,ik)         
            tra(ii,ij,ik,6) = ( tra(ii,ij,ik,1) + zwgt * ( dta%alk(ib,ik) - tra(ii,ij,ik,6) ) ) * tmask(ii,ij,ik)         
         END DO
      END DO 
      !
      IF( kt .eq. nit000 ) CLOSE( unit = 102 )
      !
      IF( nn_timing == 1 ) CALL timing_stop('bdy_trc_frs')
      !
   END SUBROUTINE bdy_trc_frs
  
   SUBROUTINE bdy_trc_spe( idx, dta, kt )
      !!----------------------------------------------------------------------
      !!                 ***  SUBROUTINE bdy_trc_frs  ***
      !!                    
      !! ** Purpose : Apply a specified value for tracers at open boundaries.
      !! 
      !!----------------------------------------------------------------------
      INTEGER,         INTENT(in) ::   kt
      TYPE(OBC_INDEX), INTENT(in) ::   idx  ! OBC indices
      TYPE(OBC_DATA),  INTENT(in) ::   dta  ! OBC external data
      !! 
      REAL(wp) ::   zwgt           ! boundary weight
      INTEGER  ::   ib, ik, igrd   ! dummy loop indices
      INTEGER  ::   ii, ij         ! 2D addresses
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 ) CALL timing_start('bdy_trc_spe')
      !
      igrd = 1                       ! Everything is at T-points here
      DO ib = 1, idx%nblenrim(igrd)
         ii = idx%nbi(ib,igrd)
         ij = idx%nbj(ib,igrd)
         DO ik = 1, jpkm1
            tra(ii,ij,ik,1) = dta%po4(ib,ik) * tmask(ii,ij,ik)
            tra(ii,ij,ik,2) = dta%dop(ib,ik) * tmask(ii,ij,ik)
            tra(ii,ij,ik,3) = dta%fed(ib,ik) * tmask(ii,ij,ik)
            tra(ii,ij,ik,4) = dta%oxy(ib,ik) * tmask(ii,ij,ik)
            tra(ii,ij,ik,5) = dta%dic(ib,ik) * tmask(ii,ij,ik)
            tra(ii,ij,ik,6) = dta%alk(ib,ik) * tmask(ii,ij,ik)
         END DO
      END DO
      !
      IF( kt .eq. nit000 ) CLOSE( unit = 102 )
      !
      IF( nn_timing == 1 ) CALL timing_stop('bdy_trc_spe')
      !
   END SUBROUTINE bdy_trc_spe

   SUBROUTINE bdy_trc_nmn( idx, dta, kt )
      !!----------------------------------------------------------------------
      !!                 ***  SUBROUTINE bdy_trc_nmn  ***
      !!                    
      !! ** Purpose : Duplicate the value for tracers at open boundaries.
      !! 
      !!----------------------------------------------------------------------
      INTEGER,         INTENT(in) ::   kt
      TYPE(OBC_INDEX), INTENT(in) ::   idx  ! OBC indices
      TYPE(OBC_DATA),  INTENT(in) ::   dta  ! OBC external data
      !! 
      REAL(wp) ::   zwgt           ! boundary weight
      INTEGER  ::   ib, ik, igrd, jn
      INTEGER  ::   ii, ij,zcoef, zcoef1,zcoef2, ip, jp   ! 2D addresses
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 ) CALL timing_start('bdy_trc_nmn')
      !
      igrd = 1                       ! Everything is at T-points here
      DO ib = 1, idx%nblenrim(igrd)
         ii = idx%nbi(ib,igrd)
         ij = idx%nbj(ib,igrd)
         DO ik = 1, jpkm1
            ! search the sense of the gradient
            zcoef1 = bdytmask(ii-1,ij  ) +  bdytmask(ii+1,ij  )
            zcoef2 = bdytmask(ii  ,ij-1) +  bdytmask(ii  ,ij+1)
            IF ( zcoef1+zcoef2 == 0) THEN
               ! corner
               DO jn=1,6
                  zcoef = tmask(ii-1,ij,ik) + tmask(ii+1,ij,ik) +  tmask(ii,ij-1,ik) +  tmask(ii,ij+1,ik)
                  tra(ii,ij,ik,jn) = tra(ii-1,ij  ,ik,jn) * tmask(ii-1,ij  ,ik) + &
                    &                    tra(ii+1,ij  ,ik,jn) * tmask(ii+1,ij  ,ik) + &
                    &                    tra(ii  ,ij-1,ik,jn) * tmask(ii  ,ij-1,ik) + &
                    &                    tra(ii  ,ij+1,ik,jn) * tmask(ii  ,ij+1,ik)
                  tra(ii,ij,ik,jn) = ( tra(ii,ij,ik,jn) / MAX( 1, zcoef) ) * tmask(ii,ij,ik)
               ENDDO
            ELSE
               ip = bdytmask(ii+1,ij  ) - bdytmask(ii-1,ij  )
               jp = bdytmask(ii  ,ij+1) - bdytmask(ii  ,ij-1)
               DO jn=1,6
                  tra(ii,ij,ik,jn) = tra(ii+ip,ij+jp,ik,jn) * tmask(ii+ip,ij+jp,ik)
               ENDDO
            ENDIF
         END DO
      END DO
      !
      IF( kt .eq. nit000 ) CLOSE( unit = 102 )
      !
      IF( nn_timing == 1 ) CALL timing_stop('bdy_trc_nmn')
      !
   END SUBROUTINE bdy_trc_nmn
 

   SUBROUTINE bdy_trc_orlanski( idx, dta, ll_npo )
      !!----------------------------------------------------------------------
      !!                 ***  SUBROUTINE bdy_trc_orlanski  ***
      !!             
      !!              - Apply Orlanski radiation to top tracers. 
      !!              - Wrapper routine for bdy_orlanski_3d
      !! 
      !!
      !! References:  Marchesiello, McWilliams and Shchepetkin, Ocean Modelling vol. 3 (2001)    
      !!----------------------------------------------------------------------
      TYPE(OBC_INDEX),              INTENT(in) ::   idx  ! OBC indices
      TYPE(OBC_DATA),               INTENT(in) ::   dta  ! OBC external data
      LOGICAL,                      INTENT(in) ::   ll_npo  ! switch for NPO version

      INTEGER  ::   igrd, jn                                ! grid index
      !!----------------------------------------------------------------------

      IF( nn_timing == 1 ) CALL timing_start('bdy_trc_orlanski')
      !
      igrd = 1      ! Orlanski bc on temperature; 
      !
      CALL bdy_orlanski_3d( idx, igrd, trb(:,:,:,1), tra(:,:,:,1), dta%po4, ll_npo )
      CALL bdy_orlanski_3d( idx, igrd, trb(:,:,:,2), tra(:,:,:,2), dta%dop, ll_npo )
      CALL bdy_orlanski_3d( idx, igrd, trb(:,:,:,3), tra(:,:,:,3), dta%fed, ll_npo )
      CALL bdy_orlanski_3d( idx, igrd, trb(:,:,:,4), tra(:,:,:,4), dta%oxy, ll_npo )
      CALL bdy_orlanski_3d( idx, igrd, trb(:,:,:,5), tra(:,:,:,5), dta%dic, ll_npo )
      CALL bdy_orlanski_3d( idx, igrd, trb(:,:,:,6), tra(:,:,:,6), dta%alk, ll_npo )

      IF( nn_timing == 1 ) CALL timing_stop('bdy_trc_orlanski')
      !

   END SUBROUTINE bdy_trc_orlanski


   SUBROUTINE bdy_trc_rnf( idx, dta, kt )
      !!----------------------------------------------------------------------
      !!                 ***  SUBROUTINE bdy_tca_rnf  ***
      !!                    
      !! ** Purpose : Apply the runoff values for tracers at open boundaries:
      !!                  - specified to 0.1 PSU for the salinity
      !!                  - duplicate the value for the temperature
      !! 
      !!----------------------------------------------------------------------
      INTEGER,         INTENT(in) ::   kt
      TYPE(OBC_INDEX), INTENT(in) ::   idx  ! OBC indices
      TYPE(OBC_DATA),  INTENT(in) ::   dta  ! OBC external data
      !! 
      REAL(wp) ::   zwgt           ! boundary weight
      INTEGER  ::   ib, ik, igrd   ! dummy loop indices
      INTEGER  ::   ii, ij, ip, jp ! 2D addresses
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 ) CALL timing_start('bdy_trc_rnf')
      !
      igrd = 1                       ! Everything is at T-points here
      DO ib = 1, idx%nblenrim(igrd)
         ii = idx%nbi(ib,igrd)
         ij = idx%nbj(ib,igrd)
         DO ik = 1, jpkm1
            ip = bdytmask(ii+1,ij  ) - bdytmask(ii-1,ij  )
            jp = bdytmask(ii  ,ij+1) - bdytmask(ii  ,ij-1)
            !tsa(ii,ij,ik,jp_tem) = tsa(ii+ip,ij+jp,ik,jp_tem) * tmask(ii,ij,ik)
            !tsa(ii,ij,ik,jp_sal) =                        0.1 * tmask(ii,ij,ik)
            ! consider runoff tracer here
         END DO
      END DO
      !
      IF( kt .eq. nit000 ) CLOSE( unit = 102 )
      !
      IF( nn_timing == 1 ) CALL timing_stop('bdy_trc_rnf')
      !
   END SUBROUTINE bdy_trc_rnf

   SUBROUTINE bdy_trc_dmp( kt )
      !!----------------------------------------------------------------------
      !!                 ***  SUBROUTINE bdy_trc_dmp  ***
      !!                    
      !! ** Purpose : Apply damping for tracers at open boundaries.
      !! 
      !!----------------------------------------------------------------------
      INTEGER,         INTENT(in) ::   kt
      !! 
      REAL(wp) ::   zwgt           ! boundary weight
      REAL(wp) ::   zta, zsa, ztime
      INTEGER  ::   ib, ik, igrd   ! dummy loop indices
      INTEGER  ::   ii, ij         ! 2D addresses
      INTEGER  ::   ib_bdy         ! Loop index
      !!----------------------------------------------------------------------
      !
      WRITE(*,*) '**********************HERE *****************************'
      IF( nn_timing == 1 ) CALL timing_start('bdy_trc_dmp')
      !
      DO ib_bdy=1, nb_bdy
         IF ( ln_trc_dmp(ib_bdy) ) THEN
            igrd = 1                       ! Everything is at T-points here
            DO ib = 1, idx_bdy(ib_bdy)%nblen(igrd)
               ii = idx_bdy(ib_bdy)%nbi(ib,igrd)
               ij = idx_bdy(ib_bdy)%nbj(ib,igrd)
               zwgt = idx_bdy(ib_bdy)%nbd(ib,igrd)
               DO ik = 1, jpkm1
                  !zta = zwgt * ( dta_bdy(ib_bdy)%tem(ib,ik) - tsb(ii,ij,ik,jp_tem) ) * tmask(ii,ij,ik)
                  !tsa(ii,ij,ik,jp_tem) = tsa(ii,ij,ik,jp_tem) + zta
                  !zsa = zwgt * ( dta_bdy(ib_bdy)%sal(ib,ik) - tsb(ii,ij,ik,jp_sal) ) * tmask(ii,ij,ik)
                  !tsa(ii,ij,ik,jp_sal) = tsa(ii,ij,ik,jp_sal) + zsa
                  ! consider tracer here
                  zsa = zwgt * ( dta_bdy(ib_bdy)%po4(ib,ik) - trb(ii,ij,ik,1) ) * tmask(ii,ij,ik)
                  tra(ii,ij,ik,1) = tra(ii,ij,ik,1) + zsa

                  zsa = zwgt * ( dta_bdy(ib_bdy)%dop(ib,ik) - trb(ii,ij,ik,2) ) * tmask(ii,ij,ik)
                  tra(ii,ij,ik,2) = tra(ii,ij,ik,2) + zsa

                  zsa = zwgt * ( dta_bdy(ib_bdy)%fed(ib,ik) - trb(ii,ij,ik,3) ) * tmask(ii,ij,ik)
                  tra(ii,ij,ik,3) = tra(ii,ij,ik,3) + zsa

                  zsa = zwgt * ( dta_bdy(ib_bdy)%oxy(ib,ik) - trb(ii,ij,ik,4) ) * tmask(ii,ij,ik)
                  tra(ii,ij,ik,4) = tra(ii,ij,ik,4) + zsa

                  zsa = zwgt * ( dta_bdy(ib_bdy)%dic(ib,ik) - trb(ii,ij,ik,5) ) * tmask(ii,ij,ik)
                  tra(ii,ij,ik,5) = tra(ii,ij,ik,5) + zsa

                  zsa = zwgt * ( dta_bdy(ib_bdy)%alk(ib,ik) - trb(ii,ij,ik,4) ) * tmask(ii,ij,ik)
                  tra(ii,ij,ik,6) = tra(ii,ij,ik,6) + zsa
               END DO
            END DO
         ENDIF
      ENDDO
      !
      IF( nn_timing == 1 ) CALL timing_stop('bdy_trc_dmp')
      !
   END SUBROUTINE bdy_trc_dmp
 
#else
   !!----------------------------------------------------------------------
   !!   Dummy module                   NO Unstruct Open Boundary Conditions
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE bdy_trc(kt)      ! Empty routine
      WRITE(*,*) 'bdy_trc: You should not have seen this print! error?', kt
   END SUBROUTINE bdy_trc

   SUBROUTINE bdy_trc_dmp(kt)      ! Empty routine
      WRITE(*,*) 'bdy_trc_dmp: You should not have seen this print! error?', kt
   END SUBROUTINE bdy_trc_dmp

#endif

   !!======================================================================
END MODULE bdytrc
