MODULE zdfmxl
   !!======================================================================
   !!                       ***  MODULE  zdfmxl  ***
   !! Ocean physics: mixed layer depth 
   !!======================================================================
   !! History :  1.0  ! 2003-08  (G. Madec)  original code
   !!            3.2  ! 2009-07  (S. Masson, G. Madec)  IOM + merge of DO-loop
   !!            3.7  ! 2012-03  (G. Madec)  make public the density criteria for trdmxl 
   !!             -   ! 2014-02  (F. Roquet)  mixed layer depth calculated using N2 instead of rhop 
   !!----------------------------------------------------------------------
   !!   zdf_mxl      : Compute the turbocline and mixed layer depths.
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers variables
   USE dom_oce         ! ocean space and time domain variables
   USE zdf_oce         ! ocean vertical physics
   USE in_out_manager  ! I/O manager
   USE prtctl          ! Print control
   USE phycst          ! physical constants
   USE iom             ! I/O library
   USE lib_mpp         ! MPP library
   USE wrk_nemo        ! work arrays
   USE timing          ! Timing
   USE trc_oce, ONLY : lk_offline ! offline flag

   IMPLICIT NONE
   PRIVATE

   PUBLIC   zdf_mxl       ! called by step.F90
   PUBLIC   zdf_mxl_alloc ! Used in zdf_tke_init

   INTEGER , PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) ::   nmln    !: number of level in the mixed layer (used by TOP)
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) ::   hmld    !: mixing layer depth (turbocline)      [m]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) ::   hmlp    !: mixed layer depth  (rho=rho0+zdcrit) [m]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) ::   hmlpt   !: mixed layer depth at t-points        [m]
#if defined key_MLD
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) ::   hmlpts    !: mixed layer depth using temperature/salinity profile        [m]
   REAL(wp), PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) ::   hmlptsave    !: MLD using T/S profile       [m]!
   INTEGER , PUBLIC, ALLOCATABLE, SAVE, DIMENSION(:,:) ::   nmlnsave     ! : key MLD save the bottom level of Mixed layer for tracer work
   INTEGER , PUBLIC, SAVE :: MLDCount=0 ! counter
#endif

   REAL(wp), PUBLIC ::   rho_c = 0.01_wp    !: density criterion for mixed layer depth
   REAL(wp)         ::   avt_c = 5.e-4_wp   ! Kz criterion for the turbocline depth

   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2011)
   !! $Id: zdfmxl.F90 6353 2016-02-24 19:04:41Z mathiot $ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   INTEGER FUNCTION zdf_mxl_alloc()
      !!----------------------------------------------------------------------
      !!               ***  FUNCTION zdf_mxl_alloc  ***
      !!----------------------------------------------------------------------
      zdf_mxl_alloc = 0      ! set to zero if no array to be allocated
      IF( .NOT. ALLOCATED( nmln ) ) THEN
         ALLOCATE( nmln(jpi,jpj), hmld(jpi,jpj), hmlp(jpi,jpj), hmlpt(jpi,jpj), STAT= zdf_mxl_alloc )
#if defined key_MLD
         ALLOCATE(hmlpts(jpi,jpj), hmlptsave(jpi,jpj), nmlnsave(jpi,jpj))
         hmlptsave(:,:)=0.0_wp
         nmlnsave(:,:)=0
#endif

         !
         IF( lk_mpp             )   CALL mpp_sum ( zdf_mxl_alloc )
         IF( zdf_mxl_alloc /= 0 )   CALL ctl_warn('zdf_mxl_alloc: failed to allocate arrays.')
         !
      ENDIF
   END FUNCTION zdf_mxl_alloc


   SUBROUTINE zdf_mxl( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE zdfmxl  ***
      !!                   
      !! ** Purpose :   Compute the turbocline depth and the mixed layer depth
      !!              with density criteria.
      !!
      !! ** Method  :   The mixed layer depth is the shallowest W depth with 
      !!      the density of the corresponding T point (just bellow) bellow a
      !!      given value defined locally as rho(10m) + rho_c
      !!               The turbocline depth is the depth at which the vertical
      !!      eddy diffusivity coefficient (resulting from the vertical physics
      !!      alone, not the isopycnal part, see trazdf.F) fall below a given
      !!      value defined locally (avt_c here taken equal to 5 cm/s2 by default)
      !!
      !! ** Action  :   nmln, hmld, hmlp, hmlpt
      !! ** Edit: April 2016- added key_MLD code to allow for gradient T/S
      !!         determination for MLD. Pennelly, Univerity of Alberta
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt   ! ocean time-step index
      !
      INTEGER  ::   ji, jj, jk, nmln2      ! dummy loop indices
      INTEGER  ::   iikn, iiki, ikt ! local integer
      REAL(wp) ::   zN2_c           ! local scalar
      INTEGER, POINTER, DIMENSION(:,:) ::   imld   ! 2D workspace
#if defined key_MLD
      REAL(wp), DIMENSION(:), ALLOCATABLE :: ltempfit, lsalfit, dtmaxfit, dsmaxfit
      REAL(wp), DIMENSION(:), ALLOCATABLE :: error_T, error_S, P, E
      REAL(wp), DIMENSION(:), ALLOCATABLE :: polyfitans ! Function
      REAL(wp), DIMENSION(:), ALLOCATABLE :: polyvalans ! Function
      REAL(wp), DIMENSION(2)              :: At, As, var_temp, var_salt, var_Tfit, var_Sfit
      REAL(wp) :: upperdsmax, upperdtmin,tmin, smin,  DEG2RAD, C1, XX
      REAL(wp) ::   MLDthreshold=300.0 ! Don't change MLD values which are less than this
      INTEGER  ::   ind,i  ! dummy loop index
      REAL(wp) ::   errortol=1e-8
      INTEGER  ::   kk=5 ! Starting level for calculations. Top levels cause issues
      INTEGER  ::   indexsize ! number of points being calculated
      INTEGER  ::   m, k, mld_fittemp, mld_fitsalt, ind_E, mldbotT, mldbotS
#endif

      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('zdf_mxl')
      !
      CALL wrk_alloc( jpi,jpj, imld )

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'zdf_mxl : mixed layer depth'
         IF(lwp) WRITE(numout,*) '~~~~~~~ '
         !                             ! allocate zdfmxl arrays
         IF( zdf_mxl_alloc() /= 0 )   CALL ctl_stop( 'STOP', 'zdf_mxl : unable to allocate arrays' )
      ENDIF

      ! w-level of the mixing and mixed layers
      nmln(:,:)  = nlb10               ! Initialization to the number of w ocean point
      hmlp(:,:)  = 0._wp               ! here hmlp used as a dummy variable, integrating vertically N^2
      zN2_c = grav * rho_c * r1_rau0   ! convert density criteria into N^2 criteria
      DO jk = nlb10, jpkm1
         DO jj = 1, jpj                ! Mixed layer level: w-level 
            DO ji = 1, jpi
               ikt = mbkt(ji,jj)
               hmlp(ji,jj) = hmlp(ji,jj) + MAX( rn2b(ji,jj,jk) , 0._wp ) * fse3w(ji,jj,jk)
               IF( hmlp(ji,jj) < zN2_c )   nmln(ji,jj) = MIN( jk , ikt ) + 1   ! Mixed layer level
            END DO
         END DO
      END DO
      !
      ! w-level of the turbocline
      imld(:,:) = mbkt(:,:) + 1        ! Initialization to the number of w ocean point
      DO jk = jpkm1, nlb10, -1         ! from the bottom to nlb10 
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( avt (ji,jj,jk) < avt_c * wmask(ji,jj,jk) )   imld(ji,jj) = jk      ! Turbocline 
            END DO
         END DO
      END DO
      ! depth of the mixing and mixed layers
      DO jj = 1, jpj
         DO ji = 1, jpi
            iiki = imld(ji,jj)
            iikn = nmln(ji,jj)
            hmld (ji,jj) = fsdepw(ji,jj,iiki  ) * ssmask(ji,jj)    ! Turbocline depth 
            hmlp (ji,jj) = fsdepw(ji,jj,iikn  ) * ssmask(ji,jj)    ! Mixed layer depth
            hmlpt(ji,jj) = fsdept(ji,jj,iikn-1) * ssmask(ji,jj)    ! depth of the last T-point inside the mixed layer
         END DO
      END DO


! Change the MLD since the density gradient method above is not that great for
! deep mixed layers
#if defined key_MLD
      if(MLDCount.LT.10 .OR. mod(kt-1,10)==0 ) THEN! Do MLD every 10 timesteps to save time, else reuse MLD values
        MLDCount=MLDCount+1
!         IF(lwp) WRITE(numout,*) 'MLDcount is : ',MLDCount     ! ssh statistics

!    if(kt>0) THEN
!      elseif(mod(kt-1,10)==0 .OR. kt==1) THEN   ! do MLD every 10 timesteps to save time. inbetween, just re-use fixed MLD values.
      DEG2RAD = (4.D0*DATAN(1.D0))/180
      hmlpts=hmlp
      DO ji=1,jpi
         DO jj=1,jpj
            if(hmlpts(ji,jj).LT.MLDthreshold) THEN  ! MLD is less than our threshold, do nothing
            else                                  ! MLD is deeper than our threshold, determine new MLD
               ind=count(tsn(ji,jj,:,jp_sal).GT.0.0)
               indexsize=ind-kk+1 ! Vertical points to be used for calculations
               ALLOCATE(P(indexsize))
               XX=sin(abs(gphit(ji,jj)*DEG2RAD)) ! need to get pressure- determine from latitude and depth
               C1=5.92E-3+XX**2*5.25E-3
               P=((1.0-C1)-sqrt(((1.0-C1)**2.0)-(8.84E-6*gdept_0(ji,jj,kk:ind))))/4.42E-6
! get pressure
               m=size(tsn(ji,jj,kk:ind,jp_tem)) ! number of vertical points being used to do calculations
               ALLOCATE(error_T(m), error_S(m))
               ALLOCATE(ltempfit(indexsize), lsalfit(indexsize))
               error_T(1)=0.0_wp
               error_S(1)=0.0_wp
               ltempfit(:)=0.0_wp
               lsalfit(:)=0.0_wp
               do k=2,m
                  CALL polyfit(P(1:k),tsn(ji,jj,kk:(kk+k-1),jp_tem),polyfitans)
                  At=polyfitans
                  deallocate(polyfitans)
                  CALL polyval(At,P(1:k),polyvalans)
                  ltempfit(1:k)=polyvalans
                  deallocate(polyvalans)
                  CALL polyfit(P(1:k),tsn(ji,jj,kk:(kk+k-1),jp_sal),polyfitans)
                  As=polyfitans
                  deallocate(polyfitans)
                  call polyval(As,P(1:k),polyvalans)
                  lsalfit(1:k)=polyvalans
                  deallocate(polyvalans)
                  error_T(k)=dot_product(tsn(ji,jj,kk:(kk+k-1),jp_tem)-ltempfit(1:k),(tsn(ji,jj,kk:(kk+k-1),jp_tem)-ltempfit(1:k)))
                  error_S(k)=dot_product(tsn(ji,jj,kk:(kk+k-1),jp_sal)-lsalfit(1:k),(tsn(ji,jj,kk:(kk+k-1),jp_sal)-lsalfit(1:k)))
               end do
! normalize the errors:
               error_T = error_T/(SUM(error_T))
               error_S = error_S/(SUM(error_S))
               mld_fittemp = count(error_T.LT.errortol) ! determine how many vertical points are within our error threshold, errortol
               mld_fitsalt = count(error_S.LT.errortol)
! exted the mixed layer fit to the depth of the profile
               ltempfit(:)=0.0_wp
               lsalfit(:)=0.0_wp
               CALL polyfit(P(1:mld_fittemp),tsn(ji,jj,kk:(kk+mld_fittemp-1),jp_tem),polyfitans)
               var_temp = polyfitans
               deallocate(polyfitans)
               CALL polyval(var_temp,P,polyvalans)
               ltempfit = polyvalans
              deallocate(polyvalans)
               CALL polyfit(P(1:mld_fitsalt),tsn(ji,jj,kk:(kk+mld_fitsalt-1),jp_sal),polyfitans)
               var_salt = polyfitans
               deallocate(polyfitans)
               CALL polyval(var_salt,P,polyvalans)
               lsalfit= polyvalans
               deallocate(polyvalans)
               ALLOCATE(E(size(ltempfit)))
               E=0.0_wp
               E = ltempfit-tsn(ji,jj,kk:kk+indexsize-1,jp_tem)
               ind_E=0
               do i=1,indexsize
                  if(ABS(E(i)).GT.0.05) then
                     ind_E=i
                     exit
                  endif
               end do
               ALLOCATE(dtmaxfit(indexsize), dsmaxfit(indexsize))
               dtmaxfit=0.0_wp
               dsmaxfit=0.0_wp
               var_Tfit(1:2)=100.0_wp
               IF (ind_E.GT.0 .AND. ind_E .GT.1) THEN
                  IF(ind_E .LT. SIZE(tsn(ji,jj,kk:kk+indexsize-1,jp_tem))) then
                     IF(tsn(ji,jj,kk+ind_E,jp_tem) .EQ. MAXVAL(tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_tem)) .OR. tsn(ji,jj,kk+ind_E,jp_tem) .EQ. MINVAL(tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_tem))) THEN
                        CALL polyfit(P(ind_E-1:ind_E),tsn(ji,jj,kk+ind_E-1:kk+ind_E,jp_tem),polyfitans)
                        var_Tfit = polyfitans
                        deallocate(polyfitans)
                     else
                        CALL polyfit(P(ind_E-1:ind_E+1),tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_tem),polyfitans)
                        var_Tfit = polyfitans
                        deallocate(polyfitans)
                     endif
                  else
                     CALL polyfit(P(ind_E-1:ind_E),tsn(ji,jj,kk+ind_E-1:kk+ind_E,jp_tem),polyfitans)
                     var_Tfit = polyfitans
                     deallocate(polyfitans)
                  ENDIF
                  CALL polyval(var_Tfit,P,polyvalans)
                  dtmaxfit = polyvalans
                  deallocate(polyvalans)
               ENDIF
               if(abs(var_Tfit(2)).LT.abs(var_temp(2))) THEN
                  ltempfit = dtmaxfit
                  var_Tfit=0
                  E = ltempfit-tsn(ji,jj,kk:kk+indexsize-1,jp_tem)
                  DO i=1,indexsize
                     IF(ABS(E(i)).GT.0.05_wp) THEN
                        ind_E=i
                        exit
                     ENDIF
                  END DO
                  IF (ind_E.GT.0 .AND. ind_E .GT.1) THEN
                     IF(ind_E .LT. SIZE(tsn(ji,jj,kk:kk+indexsize-1,jp_tem))) THEN
                        IF(tsn(ji,jj,kk+ind_E,jp_tem) .EQ. MAXVAL(tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_tem)) .OR. tsn(ji,jj,kk+ind_E,jp_tem) .EQ. MINVAL(tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_tem))) THEN
                           CALL polyfit(P(ind_E-1:ind_E),tsn(ji,jj,kk+ind_E-1:kk+ind_E,jp_tem),polyfitans)
                           var_Tfit=polyfitans
                           deallocate(polyfitans)
                        ELSE
                           CALL polyfit(P(ind_E-1:ind_E+1),tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_tem),polyfitans)
                           var_Tfit = polyfitans
                           deallocate(polyfitans)
                        ENDIF
                     ELSE
                        CALL polyfit(P(ind_E-1:ind_E),tsn(ji,jj,kk+ind_E-1:kk+ind_E,jp_tem),polyfitans)
                        var_Tfit = polyfitans
                        deallocate(polyfitans)
                     ENDIF
                     CALL polyval(var_Tfit,P,polyvalans)
                     dtmaxfit = polyvalans
                     deallocate(polyvalans)
                  ENDIF
               ENDIF
               E= lsalfit-tsn(ji,jj,kk:kk+indexsize-1,jp_sal)
               DO i=1,indexsize
                  IF(ABS(E(i)).GT.0.002_wp) THEN
                     ind_E=i
                     exit
                  ENDIF
               END DO
               var_Sfit(1:2)=100_wp
               IF (ind_E.GT.0 .AND. ind_E .GT.1) THEN
                  IF(ind_E .LT. SIZE(tsn(ji,jj,kk:kk+indexsize-1,jp_sal))) THEN
                     IF(tsn(ji,jj,kk+ind_E,jp_sal) .EQ. MAXVAL(tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_sal)) .OR. tsn(ji,jj,kk+ind_E,jp_sal) .EQ. MINVAL(tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_sal))) THEN
                        CALL polyfit(P(ind_E-1:ind_E),tsn(ji,jj,kk+ind_E-1:kk+ind_E,jp_sal),polyfitans)
                        var_Sfit = polyfitans
                        deallocate(polyfitans)
                     ELSE
                        CALL polyfit(P(ind_E-1:ind_E+1),tsn(ji,jj,kk+ind_E-1:kk+ind_E+1,jp_sal),polyfitans)
                        var_Sfit = polyfitans
                        deallocate(polyfitans)
                     ENDIF
                  ELSE
                     CALL polyfit(P(ind_E-1:ind_E),tsn(ji,jj,kk+ind_E-1:kk+ind_E,jp_sal),polyfitans)
                     var_Sfit = polyfitans
                     deallocate(polyfitans)
                  ENDIF
                  CALL polyval(var_Sfit,P,polyvalans)
                  dsmaxfit = polyvalans
                  deallocate(polyvalans)
               ENDIF

               upperdtmin=0.0_wp
               upperdsmax=0.0_wp
               mldbotT=0.0_wp
               mldbotS=0.0_wp
               tmin=minval(abs(dtmaxfit-ltempfit))
               smin=minval(abs(dsmaxfit-lsalfit))
               do i=1,indexsize
                  if(abs(dtmaxfit(i)-ltempfit(i)).EQ.tmin) then
                     upperdtmin=i
                     mldbotT=i
                  endif
                  if(abs(dsmaxfit(i)-lsalfit(i)).EQ.smin) then
                     upperdsmax=i
                     mldbotS=i
                  endif
               ENDDO

               if(all(dtmaxfit-ltempfit.GT.0.0_wp).EQV..TRUE.) then
                  upperdtmin =0.0_wp
               endif
               if(all(-dtmaxfit+ltempfit.GT.0.0_wp).EQV..TRUE.) then
                  upperdtmin=0.0_wp
               endif
               if (upperdtmin.NE.0.0_wp) then
                  upperdtmin = gdept_0(ji,jj,kk+upperdtmin)
               endif

               if (all(dsmaxfit-lsalfit.GT.0.0_wp).EQV..TRUE.) then
                  upperdsmax =0.0_wp
               endif
               if (all(-dsmaxfit+lsalfit.GT.0.0_wp).EQV..TRUE.) then
                  upperdsmax=0.0_wp
               endif
               if (upperdsmax.NE.0.0_wp) then
                  upperdsmax = gdept_0(ji,jj,kk+upperdsmax)
               endif

               if(upperdtmin.NE.0.0_wp .AND. upperdtmin.LT.hmlpts(ji,jj)) then
                  hmlpts(ji,jj)= upperdtmin
                  nmln(ji,jj)  = mldbotT ! mixed layer T   CP
                  hmlpt(ji,jj) = fsdept(ji,jj,max(nmln(ji,jj)-1,1))
               else if (upperdsmax.NE.0.0_wp .AND. upperdsmax.LT.hmlpts(ji,jj)) then
                  hmlpts(ji,jj)= upperdsmax
                  nmln(ji,jj)  = mldbotS ! mixed layer S   CP
                  hmlpt(ji,jj) = fsdept(ji,jj,max(nmln(ji,jj)-1,1))
               endif
               DEALLOCATE(error_T, error_S)
               DEALLOCATE(ltempfit, lsalfit)
               DEALLOCATE(dtmaxfit, E, dsmaxfit)
               DEALLOCATE(P)
            ENDIF ! Threshoold logic check loop
         END DO ! jpj loop
      END DO    ! jpi loop
! end of MLD fix: key_MLD    CP
     hmlptsave=hmlpts
!     if(.NOT.allocated(nmlnsave)) ALLOCATE(nmlnsave(jpi,jpj))
     nmlnsave=nmln
   ELSE  ! not a modulus of the timestep =5?
     hmlpts=hmlptsave
     if(SUM(nmlnsave(:,:))==0) THEN
        nmlnsave=nmln
     endif
        nmln=nmlnsave
ENDIF

#endif
!         nmln2 = SUM( nmln(:,:) )
!         IF( lk_mpp )   CALL mpp_sum( nmln2 )      ! sum over the global domain
!         IF(lwp) WRITE(numout,9333) kt, nmln2     ! ssh statistics
!9333  FORMAT(' kt=:', i8, ' nmln sum : ', i10)


      ! no need to output in offline mode
      IF( .NOT.lk_offline ) THEN   
         IF ( iom_use("mldr10_1") ) THEN
            IF( ln_isfcav ) THEN
               CALL iom_put( "mldr10_1", hmlp - risfdep)   ! mixed layer thickness
            ELSE
               CALL iom_put( "mldr10_1", hmlp )            ! mixed layer depth
            END IF
         END IF
         IF ( iom_use("mldkz5") ) THEN
            IF( ln_isfcav ) THEN
               CALL iom_put( "mldkz5"  , hmld - risfdep )   ! turbocline thickness
            ELSE
               CALL iom_put( "mldkz5"  , hmld )             ! turbocline depth
            END IF
         END IF
         IF ( iom_use("somxlts") ) THEN
            IF ( ln_isfcav ) THEN
               CALL iom_put( "somxlts", hmlpts - risfdep )    ! updated mixed layer thickness (CP)
            ELSE
               CALL iom_put( "somxlts", hmlpts )              ! updated mixed layer depth (CP)
            ENDIF 
         ENDIF
      ENDIF
      
      IF(ln_ctl)   CALL prt_ctl( tab2d_1=REAL(nmln,wp), clinfo1=' nmln : ', tab2d_2=hmlp, clinfo2=' hmlp : ', ovlap=1 )
      !
      CALL wrk_dealloc( jpi,jpj, imld )
      !
      IF( nn_timing == 1 )  CALL timing_stop('zdf_mxl')
      !
   END SUBROUTINE zdf_mxl

#if defined key_MLD
SUBROUTINE polyfit(vx,vy,polyfitanss)
    implicit none
    real(wp), dimension(:), intent(in)    :: vx, vy
    real(wp), dimension(:), ALLOCATABLE, intent(out)   :: polyfitanss
    real(wp), dimension(:,:), allocatable :: X
    real(wp), dimension(:,:), allocatable :: XT
    real(wp), dimension(:,:), allocatable :: XTX

    integer :: i, j

    integer     :: n, lda, lwork
    integer :: info
    integer, dimension(:), allocatable :: ipiv
    real(wp), dimension(:), allocatable :: work
    allocate(polyfitanss(2))
    polyfitanss(:)=0.0_wp
    n = 2
    lda = n
    lwork = n
    allocate(ipiv(n))
    allocate(work(lwork))
    allocate(XT(n, size(vx)))
    allocate(X(size(vx), n))
    allocate(XTX(n, n))
    do i = 0, 1
       do j = 1, size(vx)
          X(j, i+1) = vx(j)**i
       end do
    end do
    XT  = transpose(X)
    XTX = matmul(XT, X)
    ! calls to LAPACK subs DGETRF and DGETRI
    call DGETRF(n, n, XTX, lda, ipiv, info)
    if ( info /= 0 ) then
       print *, "problem"
       return
    end if
    call DGETRI(n, XTX, lda, ipiv, work, lwork, info)
    if ( info /= 0 ) then
       print *, "problem"
       return
    end if
    polyfitanss = matmul( matmul(XTX, XT), vy)
    deallocate(ipiv)
    deallocate(work)
    deallocate(X)
    deallocate(XT)
    deallocate(XTX)
    return
END SUBROUTINE polyfit

SUBROUTINE polyval(vxx,vyy,polyvalanss)
    implicit none
    integer                                           :: j
    real(wp), ALLOCATABLE,dimension(:), intent(out)   :: polyvalanss
    real(wp), dimension(:), intent(in)                :: vxx, vyy

    ALLOCATE(polyvalanss(size(vyy)))
    polyvalanss(:)=0.0_wp
    do j=1,size(vyy)
       polyvalanss(j)=vxx(2)*vyy(j)+vxx(1)
    end do
    return
END SUBROUTINE polyval
#endif

   !!======================================================================
END MODULE zdfmxl
